const { v4: uuidv4 } = require('uuid');

const write = require('../../util/forum/write');
const addPostWithRetry = require('../../util/add-post-with-retry');
const callWithRetry = require('../../util/call-with-retry');
const { getBatchItems, clearBatchItems } = require('./batch-items');
const computeAuthorWeights = require('./compute-author-weights');
const { wallet, rollup } = require('../../util/contracts');
const { sendMatrixEvent } = require('../../matrix-bot');
const { stakeRollupAvailability } = require('./utils');

const submitRollup = async () => {
  const batchItems = getBatchItems();
  if (!batchItems.length) {
    return { batchItems: [] };
  }
  const authors = await computeAuthorWeights(batchItems);
  // TODO: Compute citations as aggregate of the citations of posts in the batch
  const citations = [];
  const content = `Batch of ${batchItems.length} items`;
  const embeddedData = {
    batchItems,
    nonce: uuidv4().replaceAll('-', ''),
  };
  const sender = await wallet.getAddress();
  const contentToVerify = `${content}\n\n${JSON.stringify(embeddedData, null, 2)}`;
  const signature = await wallet.signMessage(contentToVerify);
  // Write to the forum database
  const { hash: batchPostId } = await write({
    sender, authors, citations, content, embeddedData, signature,
  });
  // Add rollup post on-chain
  await addPostWithRetry(authors, batchPostId, citations);
  // Stake our availability to be the next rollup worker
  await stakeRollupAvailability();
  // Call Rollup.submitBatch
  console.log('Submitting batch', { batchPostId, batchItems, authors });
  const poolDuration = 60;
  await callWithRetry(() => rollup.submitBatch(batchPostId, batchItems, poolDuration));
  // Send matrix event
  await sendMatrixEvent('io.dgov.rollup.submit', { batchPostId, batchItems, authors });
  // Clear the batch in preparation for next batch
  await clearBatchItems(batchItems);
  return {
    batchPostId,
    batchItems,
    authors,
  };
};

module.exports = submitRollup;
